import requests
import urllib3
import re
import time
import ssl
import traceback
from requests import adapters
from .third_party.cloudscraper import cloudscraper

class TLSAdapter(adapters.HTTPAdapter):
    def init_poolmanager(self, connections, maxsize, block=False):
        ctx = ssl.create_default_context()
        ctx.set_ciphers('DEFAULT@SECLEVEL=1')
        self.poolmanager = urllib3.poolmanager.PoolManager(num_pools=connections,
                                                           maxsize=maxsize,
                                                           block=block,
                                                           ssl_version=ssl.PROTOCOL_TLSv1_2,
                                                           ssl_context=ctx)

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

def __retry(request, response, next, cfscrape, retry=0):
    if retry > 5:
        return None

    if response.status_code in [503, 429, 403]:
        if response.status_code == 503:
            time.sleep(2)
            retry = 5
        if response.status_code == 429:
            time.sleep(3)

        retry += 1
        request['validate'] = lambda response: __retry(request, response, next, cfscrape, retry)
        request['next'] = next
        request['cfscrape'] = cfscrape
        return request

def execute(request, progress=True, session=None):
    default_timeout = 10
    request.setdefault('timeout', default_timeout)

    next = request.pop('next', None)

    cfscrape = 'cfscrape' in request
    request.pop('cfscrape', None)

    validate = request.pop('validate', None)
    if not validate:
        validate = lambda response: __retry(request, response, next, cfscrape)

    if next:
        request.pop('stream', None)

    try:
        if cfscrape:
            request.pop('cfscrape', None)
            if not session:
                session = cloudscraper.create_scraper(interpreter='native')
            response = session.request(**request)
        else:
            session = requests.session()
            session.mount('https://', TLSAdapter())
            response = session.request(**request)
        exc = ''
    except:  # pragma: no cover
        try:
            if cfscrape:
                if not session:
                    session = cloudscraper.create_scraper(interpreter='native')
                response = session.request(verify=False, **request)
            else:
                response = requests.request(verify=False, **request)
            exc = ''
        except:
            exc = traceback.format_exc()
            response = lambda: None
            response.text = ''
            response.content = ''
            response.status_code = 500
    alt_request = validate(response)
    if alt_request:
        return execute(alt_request, progress)

    if next and response.status_code == 200:
        next_request = next(response)
        if next_request:
            return execute(next_request, progress, session)
        else:
            return None

    return response
